# GloFlow application and media management/publishing platform
# Copyright (C) 2021 Ivan Trajkovic
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import boto3

#-------------------------------------------------------------
def describe_instances__by_tags(p_tags_map, p_region_str):
    

    client = boto3.client("ec2", region_name=p_region_str)

    custom_filter = []
    for k, v in p_tags_map.items():
        custom_filter.append({
            "Name":   f"tag:{k}", 
            "Values": [v]
        })
    
    response = client.describe_instances(Filters=custom_filter)

    aws_instances_lst = []
    for i in response["Reservations"][0]["Instances"]:
        aws_instances_lst.append(i)

    return aws_instances_lst


# GloFlow application and media management/publishing platform
# Copyright (C) 2023 Ivan Trajkovic
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from google.cloud import bigquery

#--------------------------------------------
def table_exists(p_table_name_str,
    p_dataset_id_str,
    p_bigquery_client):

    table_ref = p_bigquery_client.dataset(p_dataset_id_str).table(p_table_name_str)
    try:
        table = p_bigquery_client.get_table(table_ref)

        # Table exists, do something...
        print(f"Table '{table_ref}' exists!")
        return True

    except Exception as e:

        if "Not found" in str(e):
            print(f"Table '{table_ref}' doesn't exist.")
            return False


# GloFlow application and media management/publishing platform
# Copyright (C) 2019 Ivan Trajkovic
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os, sys
cwd_str = os.path.abspath(os.path.dirname(__file__))

import boto3

import gf_os_aws_creds

#--------------------------------------------------
class Gf_s3_info:
    imgs__bucket            = None
    discovered_imgs__bucket = None
    s3_resource             = None

#--------------------------------------------------
# FIX!! - use gf_aws_creds.get_from_file() directly
def parse_creds(p_aws_creds_file_path_str):
    aws_s3_creds_map = gf_aws_creds.get_from_file(p_aws_creds_file_path_str)
    
    assert "AWS_ACCESS_KEY_ID" in aws_s3_creds_map.keys()
    assert "AWS_SECRET_ACCESS_KEY" in aws_s3_creds_map.keys()
    assert(len(aws_s3_creds_map["AWS_ACCESS_KEY_ID"]) == 20)
    assert(len(aws_s3_creds_map["AWS_SECRET_ACCESS_KEY"]) == 40)
    
    return aws_s3_creds_map

#---------------------------------------------------
def s3_connect(p_aws_access_key_id_str,
    p_aws_secret_access_key_str):

    session = boto3.Session(
	    aws_access_key_id     = p_aws_access_key_id_str,
	    aws_secret_access_key = p_aws_secret_access_key_str,
	)
    
    s3_resource             = session.resource("s3")
    imgs__bucket            = s3_resource.Bucket("gf--img")
    discovered_imgs__bucket = s3_resource.Bucket("gf--discovered--img")

    gf_s3_info                         = Gf_s3_info()
    gf_s3_info.imgs__bucket            = imgs__bucket
    gf_s3_info.discovered_imgs__bucket = discovered_imgs__bucket
    gf_s3_info.s3_resource             = s3_resource

    return gf_s3_info


# GloFlow application and media management/publishing platform
# Copyright (C) 2019 Ivan Trajkovic
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import time
import clint

#----------------------------------------------
def get_log_fun(p_log_fun):
	# p_log_fun("FUN_ENTER", "gf_core_logger.get_log_fun()")
	
	#----------------------------------------------	
	def log_color_display_fun(p_group, p_msg):
		
		t = str(time.time())			 
		if p_group == 'FUN_ENTER':
			clint.textui.puts(t+':'+clint.textui.colored.yellow('FUN_ENTER')+':'+p_msg)
			
		elif p_group == 'ERROR':
			clint.textui.puts(t+':'+clint.textui.colored.red(p_group)+':'+p_msg)
			
		elif p_group == 'INFO':
			clint.textui.puts(t+':'+clint.textui.colored.green(p_group)+':'+ clint.textui.colored.green(p_msg))
			
		elif p_group == 'INFO_USR':
			clint.textui.puts(t+':'+clint.textui.colored.magenta(p_group)+':'+ clint.textui.colored.magenta(p_msg))
			
		# log message is in some way related to external-systems data
		elif p_group == 'EXTERN':
			clint.textui.puts(t+':'+clint.textui.colored.blue(p_group)+':'+p_msg)
			
		# if 'TEST' is anywhere in the group string
		# elif p_group.find('TEST'):
		#	clint.textui.puts(clint.textui.colored.cyan(p_group)+':'+p_msg)
			
		#--------------
		# JAVASCRIPT FORMATING
		elif p_group == 'JS:FUN_ENTER':
			clint.textui.puts(t+':'+'       '+clint.textui.colored.yellow('FUN_ENTER')+':'+p_msg)
			
		elif p_group == 'JS:INFO':
			clint.textui.puts(t+':'+'       '+clint.textui.colored.green(p_group)+':'+ clint.textui.colored.green(p_msg))

	#----------------------------------------------
			
	return log_color_display_fun


# GloFlow application and media management/publishing platform
# Copyright (C) 2019 Ivan Trajkovic
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os, sys
modd_dir = os.path.abspath(os.path.dirname(os.path.abspath(__file__)))

from gf_core import gf_core_error

#---------------------------------------------------
#->:Map(new_image_info_map)
def verify_image_info(p_image_info_map,
	p_db_context_map,
	p_log_fun):
	p_log_fun('INFO', 'p_image_info_map:%s'%(p_image_info_map))
	assert isinstance(p_image_info_map, dict)
	
	max_title_characters = 100
	#-------------------
	# 'id_str' - is None if image_info_dict comes from the outside of the system
	#            and the ID has not yet been assigned. 
	#            ID is determined if image_info_dict comes from the DB or someplace
	#            else within the system
	
	id_str = p_image_info_map.get('id_str', None)

	#-------------------
	# TITLE
	
	title_str = None
	try:
		assert p_image_info_map.has_key('title_str')
		title_str = p_image_info_map['title_str']
		
		assert isinstance(p_image_info_map['title_str'], str)
		assert len(p_image_info_map['title_str']) < max_title_characters

	except Exception as e:
		msg_str = '''title_str [%s] is missing, invalid, or has more then max_length [%s] characters'''%(title_str, max_title_characters)
		gf_core_error.handle_exception(e,
			msg_str, #p_formated_msg_str,
			(),      #p_surrounding_context_attribs_tpl,

			p_db_context_map,
			p_log_fun,
			p_app_name_str = 'gf_image')
		
		raise Exception(msg_str)
	
	#-------------------
	# FORMAT
	
	format_str = None
	try:
		assert p_image_info_map.has_key('format_str')
		normalized_format_str = check_image_format_str(p_image_info_map['format_str'].lower(),
			                                           p_log_fun)
		format_str = normalized_format_str
			
	except Exception as e:
		msg_str = '''format_str [%s] is missing or is invalid'''%(format_str)
		
		gf_core_error.handle_exception(e,       #p_exception,
			msg_str, #p_formated_msg_str,
			(),      #p_surrounding_context_attribs_tpl,

			p_db_context_map,
			p_log_fun,
			p_app_name_str = 'gf_image')
		
		raise Exception(msg_str)
	
	#-------------------
	# WIDTH/HEIGHT
	
	width_str  = None
	height_str = None
	
	try:
		assert p_image_info_map.has_key('width_str')
		width_str = p_image_info_map['width_str']
		
		assert p_image_info_map.has_key('height_str')
		height_str = p_image_info_map['height_str']
		
		assert width_str.isdigit()
		assert height_str.isdigit()
		       
	except Exception as e:
		msg_str = '''width_str/height [%s/%s] is missing or is invalid'''%(width_str,height_str)
		
		gf_core_error.handle_exception(e,       #p_exception,
			msg_str, #p_formated_msg_str,
			(),      #p_surrounding_context_attribs_tpl,

			p_db_context_map,
			p_log_fun,
			p_app_name_str = 'gf_image')
		raise Exception(msg_str)
	
	#-------------------
	# ORIGIN URL
	
	origin_url_str = None
	
	try:
		assert p_image_info_map.has_key('origin_url_str')
		origin_url_str = p_image_info_map['origin_url_str']
	except Exception as e:
		msg_str = '''origin_url_str [%s] is missing or is invalid'''%(origin_url_str)
		
		gf_core_error.handle_exception(e,       #p_exception,
			msg_str, #p_formated_msg_str,
			(),      #p_surrounding_context_attribs_tpl,

			p_db_context_map,
			p_log_fun,
			p_app_name_str = 'gf_image')
		
		raise Exception(msg_str)
	
	#-------------------
	original_file_internal_uri_str = p_image_info_map.get('original_file_internal_uri_str',None)
	thumbnail_small_url_str        = p_image_info_map.get('thumbnail_small_url_str' ,None)
	thumbnail_medium_url_str       = p_image_info_map.get('thumbnail_medium_url_str',None)
	thumbnail_large_url_str        = p_image_info_map.get('thumbnail_large_url_str' ,None)

	#-------------------
	
	new_image_info_map = {
		'id_str': id_str,
		'title_str': title_str,
		
		'origin_url_str': origin_url_str,
		'original_file_internal_uri_str': original_file_internal_uri_str,
		
		'thumbnail_small_url_str':  thumbnail_small_url_str,
		'thumbnail_medium_url_str': thumbnail_medium_url_str,
		'thumbnail_large_url_str':  thumbnail_large_url_str,
		
		'format_str': format_str,
		'width_str':  width_str,
		'height_str': height_str,
		
		'dominant_color_hex_str':p_image_info_map['dominant_color_hex_str']
	}
	
	return new_image_info_map

#---------------------------------------------------	
#->:String(normalized_format_str)
def check_image_format_str(p_format_str, p_log_fun):
	
	normalized_format_str = None
	
	assert isinstance(p_format_str, str)
	assert p_format_str == 'jpeg' or \
		p_format_str == 'jpg'  or \
		p_format_str == 'gif'  or \
		p_format_str == 'png'
				 
	# normalize "jpg" (variation on jpeg) to "jpeg"
	if p_format_str == 'jpg':
		normalized_format_str = 'jpeg'
	else:
		normalized_format_str = p_format_str
		
	return normalized_format_str


# GloFlow application and media management/publishing platform
# Copyright (C) 2023 Ivan Trajkovic
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU G11eneral Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#---------------------------------------------------------------------------------
# IMPORTANT!! - downloads the whole sheet, all the values of columns

def get_all_columns(p_spreadsheet_id_str,
    p_subsheet_name_str,
    p_service_client):
    
    range_name_str = f"{p_subsheet_name_str}!A:Z"

    result = p_service_client.spreadsheets().values().get(spreadsheetId=p_spreadsheet_id_str,
        # body=data,
        range=range_name_str,

        # format returned data to be in column-first format 
        majorDimension='COLUMNS').execute()
    
    # iterate over each column, and get the one thats needed
    columns_vals_lst = []
    for column_vals_lst in result['values']:
        
        columns_vals_lst.append(column_vals_lst)

    return columns_vals_lst


#---------------------------------------------------------------------------------
# NAMED_RANGE

def get_named_range(p_range_name_str,
    p_spreadsheet_id_str,
    p_service_client):


    result = p_service_client.spreadsheets().values().get(spreadsheetId=p_spreadsheet_id_str,

        range=p_range_name_str,

        # format returned data to be in column-first format 
        majorDimension='COLUMNS').execute()
    
    # iterate over each column, and get the one thats needed.
    columns_vals_lst = []
    for column_vals_lst in result['values']:
        
        if len(column_vals_lst) > 0:
            columns_vals_lst.append(column_vals_lst)

    return columns_vals_lst

#---------------------------------------------------------------------------------
def get_column_by_name_from_list(p_column_name_str,
    p_columns_vals_lst,
    p_column_name__row_index_int=1):
    assert isinstance(p_columns_vals_lst, list)

    # iterate over each column, and get the one thats needed
    i=0
    for column_vals_lst in p_columns_vals_lst:

        # first test if the column has values,
        # and if it has more values than the row number where the column name is stored
        if len(column_vals_lst) > 0 and len(column_vals_lst) > (p_column_name__row_index_int+1):
            
            column_name_str = column_vals_lst[p_column_name__row_index_int]
            if column_name_str == p_column_name_str:
                return column_vals_lst, i

        i+=1

    return None, 0
    
#---------------------------------------------------------------------------------
# block is a user-created sub-section of a sheet.

def get_columns_by_name_from_named_range(p_columns_names_lst,
    p_range_name_str,
    p_spreadsheet_id_str,
    p_service_client,
    p_column_name__row_index_int=0):


    columns_lst = get_named_range(p_range_name_str,
        p_spreadsheet_id_str,
        p_service_client)


    columns_vals_lst = []
    for column_name_str in p_columns_names_lst:

        i=0
        for vals_lst in columns_lst:
             
            if len(vals_lst) > 0 and len(vals_lst) > (p_column_name__row_index_int):
                
                a_column_name_str = vals_lst[p_column_name__row_index_int]

                if column_name_str == a_column_name_str:
                    column_vals_lst = vals_lst
                    columns_vals_lst.append((column_vals_lst, i))

                    # column discovered, go to the next one
                    break

            i+=1

    return columns_vals_lst

#---------------------------------------------------------------------------------
def get_column_by_name(p_column_name_str,
    p_spreadsheet_id_str,
    p_subsheet_name_str,
    p_service_client,

    # row in which the user-titles of columns are expected
    p_column_name__row_index_int=1):


    # get all values of all the columns in a sheet
    columns_vals_lst = get_all_columns(p_spreadsheet_id_str,
        p_subsheet_name_str,
        p_service_client)

    i=0

    # iterate over each column, and get the one thats needed
    for column_vals_lst in columns_vals_lst:
        
        # first test if the column has values,
        # and if it has more values than the row number where the column name is stored
        if len(column_vals_lst) > 0 and len(column_vals_lst) > (p_column_name__row_index_int+1):
            
            column_name_str = column_vals_lst[p_column_name__row_index_int]

            if column_name_str == p_column_name_str:
                return column_vals_lst, i

        i+=1
    
    return None, 0

#---------------------------------------------------------------------------------
def get_sheet_id(p_sheet_name_str,
    p_spreadsheet_id_str,
    p_service_client):

    spreadsheet_metadata = p_service_client.spreadsheets().get(spreadsheetId=p_spreadsheet_id_str).execute()
    sheets_lst           = spreadsheet_metadata.get('sheets', '')

    for sheet in sheets_lst:
        properties = sheet.get('properties', {})
        if properties.get('title', '').lower() == p_sheet_name_str.lower():
            sheet_id_int = properties.get('sheetId', '')
            return sheet_id_int

    return None


# GloFlow application and media management/publishing platform
# Copyright (C) 2023 Ivan Trajkovic
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#---------------------------------------------------------------------------------
def search(p_username_str,
    p_service_client,
    p_results_num_int=20):

    # print("=============================>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", p_username_str)

    #--------------------
    # SEARCH
    r = p_service_client.search().list(
        q=p_username_str,
        part="id,snippet",
        maxResults=p_results_num_int
    ).execute()

    # print(r)

    total_results_for_query_int = r["pageInfo"]["totalResults"]
    items_lst = r["items"]

    #--------------------

    channels_lst = []
    videos_lst   = []

    for item_map in items_lst:

        # print(">>>>>>>>>>>>>>>>>>>", item_map)
        # print("")

        # CHANNEL
        if item_map["id"]["kind"] == "youtube#channel":
            channel_id_str          = item_map["id"]["channelId"]
            snippet_map             = item_map["snippet"]
            channel_title_str       = snippet_map["title"]
            channel_description_str = snippet_map["description"]
            thumbs_map              = snippet_map["thumbnails"]
            thumb_medium_url_str    = thumbs_map["medium"]["url"]

            '''
            print("channel ID", channel_id_str)
            print("title", channel_title_str)
            print("channel snippet", snippet_map)
            print("descr", channel_description_str)
            print("thumb medium", thumb_medium_url_str)
            '''

            channel_map = {
                "id_str":    channel_id_str,
                "title_str": channel_title_str,
                "descr_str": channel_description_str,
                "thumb_medium_url_str": thumb_medium_url_str,
            }
            channels_lst.append(channel_map)

        # VIDEO
        if item_map["id"]["kind"] == "youtube#video":
            video_id_str    = item_map["id"]["videoId"]
            snippet_map     = item_map['snippet']
            channel_id_str  = snippet_map['channelId']
            video_title_str = snippet_map['title']
            description_str = snippet_map["description"]
            thumb_medium_url_str = snippet_map["thumbnails"]["medium"]
            publish_time_str     = snippet_map["publishTime"]

            # print(f"Video ID: {video_id_str}, Video Title: {video_title_str}")

            video_map = {
                "id_str":         video_id_str,
                "channel_id_str": channel_id_str,
                "title_str":      video_title_str,
                "descr_str":      description_str,
                "thumb_medium_url_str": thumb_medium_url_str,
                "publish_time_str":     publish_time_str,
            }

            videos_lst.append(video_map)


    return channels_lst, videos_lst

#---------------------------------------------------------------------------------
def channel_info_by_username(p_username_str,
    p_service_client):

    # IMPORTANT!! - most usernames dont return any results for channels.
    #               in Youtube there is a privacy option for channel owners to disable
    #               their channel info being returned by Youtube API.
    r = p_service_client.channels().list(
            part="id,snippet,contentDetails,statistics",
            forUsername=p_username_str
        ).execute()
    print(r)

#---------------------------------------------------------------------------------
def channel_info_by_id(p_channel_id_str,
    p_service_client):

    # IMPORTANT!! - most usernames dont return any results for channels.
    #               in Youtube there is a privacy option for channel owners to disable
    #               their channel info being returned by Youtube API.
    r = p_service_client.channels().list(
            part="snippet,contentDetails,statistics",
            id=p_channel_id_str
        ).execute()

    # print(r)

    #---------------------------------------------------------------------------------
    def debug_view():
        if r['items']:
            channel_info_map = r['items'][0]
            print(f"Title: {channel_info_map['snippet']['title']}")
            print(f"Description: {channel_info_map['snippet']['description']}")
            print(f"Published at: {channel_info_map['snippet']['publishedAt']}")
            print(f"Number of subscribers: {channel_info_map['statistics']['subscriberCount']}")
            print(f"Total views: {channel_info_map['statistics']['viewCount']}")
            print(f"Total videos: {channel_info_map['statistics']['videoCount']}")
        else:
            print(f"No channel found for channel ID '{p_channel_id_str}'.")

    #---------------------------------------------------------------------------------

    debug_view()

    channel_info_map = r['items'][0]
    title_str = channel_info_map['snippet']['title']
    descr_str = channel_info_map['snippet']['description']
    published_at_str     = channel_info_map['snippet']['publishedAt']
    subscriber_count_int = channel_info_map['statistics']['subscriberCount']
    view_count_int       = channel_info_map['statistics']['viewCount']
    video_count_int      = channel_info_map['statistics']['videoCount']

    channel_meta_map = {
        "title_str": title_str,
        "descr_str": descr_str,
        "published_at_str": published_at_str,
        "subscriber_count_int": subscriber_count_int,
        "view_count_int":       view_count_int,
        "video_count_int":      video_count_int,
    }
    return channel_meta_map

#---------------------------------------------------------------------------------
def channel_videos(p_channel_id_str,
    p_service_client,
    p_results_num_int=20):

    r = p_service_client.search().list(
        channelId=p_channel_id_str,
        part="id",
        maxResults=p_results_num_int
    ).execute()

#---------------------------------------------------------------------------------
def video_info(p_video_id_str,
    p_service_client):

    r = p_service_client.videos().list(
        part='snippet,statistics',
        id=p_video_id_str
    ).execute()

    for video_result_map in r.get("items", []):
        print(f"Title: {video_result_map['snippet']['title']}")
        print(f"Thumbnail: {video_result_map['snippet']['thumbnails']['default']['url']}")
        print(f"View count: {video_result_map['statistics']['viewCount']}")


from setuptools import setup, find_packages

import os, sys
modd_str = os.path.abspath(os.path.dirname(__file__)) # module dir

with open(f'{modd_str}/requirements.txt') as f:
    required_packages = f.read().splitlines()

setup(
    name="gloflow",
    version="0.1.8",
    author="Ivan Trajkovic",
    author_email="glofloworg@gmail.com",
    description="""
Py package for interacting with the gloflow platform API's.
""",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/gloflow/gloflow",
    packages=find_packages(
        where="src",
        exclude=[
            "src/deprecated"
        ]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires=">=3.6",

    install_requires=required_packages,
    # install_requires=[
    #    "requests",
    # ],

    package_dir={"": "src"},  # Base directory for packages is src/
)



# GloFlow application and media management/publishing platform
# Copyright (C) 2020 Ivan Trajkovic
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import boto3

#--------------------------------------------------
def hosted_zone__get_id(p_name_str,
    p_aws_creds_map,
    p_region_str = "us-east-1"):
    assert isinstance(p_name_str, str)

    aws_client = boto3.client("route53",
        aws_access_key_id     = p_aws_creds_map["AWS_ACCESS_KEY_ID"],
        aws_secret_access_key = p_aws_creds_map["AWS_SECRET_ACCESS_KEY"],
        region_name           = p_region_str)

    r = aws_client.list_hosted_zones()
    
    zone_id_str = None
    for z in r["HostedZones"]:
        if f"{p_name_str}." == z["Name"]:
            zone_id_str = z["Id"]
    
    if zone_id_str == None:
        print(f"ZONE with name [{p_name_str}] doesnt exist!")

    return zone_id_str


# GloFlow application and media management/publishing platform
# Copyright (C) 2019 Ivan Trajkovic
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import json
import urllib
import requests

#---------------------------------------------------
# sends the HTTP request to the gf_images service, to process the image

#->:Map(image_processing_results_map)
def dispatch_process_extern_image(p_image_url_str,
	p_gf_images_service_host_str,
	p_log_fun,
	p_process_from_scratch_if_prexisting_bool = True):
	p_log_fun('FUN_ENTER', 'gf_images_client.dispatch_process_extern_image()')
	
	assert p_gf_images_service_host_str.startswith('http://') or \
		p_gf_images_service_host_str.startswith('https://')
				 
	# new_image_url_str = base64.b64encode(p_image_url_str)
	new_image_url_str = urllib.quote(p_image_url_str)
	url_str           = '%s/images/start_job'%(p_gf_images_service_host_str)
	headers_dict = {'accept': 'text/event-stream'} # use Server Side Events
	params_dict  = {
		'type':    'process_extern_image',
		'img_url': new_image_url_str
	}
	
	# params_dict  = {
	# 	'image_url_str':                           p_image_url_str,
	# 	'process_from_scratch_if_prexisting_bool': p_process_from_scratch_if_prexisting_str
	# }

	p_log_fun('INFO','url_str - [%s]'%(url_str))

	r = requests.get(url_str,
		params  = params_dict,
		headers = headers_dict,
		stream  = True) # HTTP Server-Side-Events
	assert r.status_code == 200

	#---------------------------------------------------
	# SSE - streaming data lines

	#->:List<:Map>
	def stream_responses():
		p_log_fun('FUN_ENTER','gf_images_client.dispatch_process_extern_image().stream_responses()')

		data_items_lst = []
		for line_str in r.iter_lines():

			p_log_fun('INFO','>>>>>>>>>>>>>>>>>>>>>>>>')
			p_log_fun('INFO',line_str)

			#filter out keep-alive new lines
			if line_str:
				if line_str.startswith('data: '):
					msg_str  = line_str.strip('data: ')
					msg_dict = json.loads(msg_str)

					assert msg_dict.has_key('status_str')
					status_str = msg_dict['status_str']

					assert isinstance(status_str, str)
					assert status_str == 'ok' or \
						   status_str == 'error'
					assert status_str == 'ok'

					data_dict = msg_dict['data_dict']
					assert isinstance(data_dict,dict)

					data_items_lst.append(data_dict)
		return data_items_lst

	#---------------------------------------------------
	data_items_lst = stream_responses()


	assert isinstance(data_items_lst,list)
	assert len(data_items_lst) == 1 #just the job_output_dict

	job_output_dict = data_items_lst[0]
	assert isinstance(job_output_dict,dict)


	p_log_fun('INFO', 'job_output_dict - %s'%(json.dumps(job_output_dict)))


	assert job_output_dict.has_key('image_id_str')
	image_id_str = job_output_dict['image_id_str']
	assert isinstance(image_id_str, str)
	
	assert job_output_dict.has_key('thumbs_info_map')
	thumbs_info_map = job_output_dict['thumbs_info_map']
	assert isinstance(thumbs_info_map, dict)

	assert thumbs_info_map.has_key('thumbnail_small_relative_url_str')
	thumbnail_small_relative_url_str = thumbs_info_map['thumbnail_small_relative_url_str']
	assert isinstance(thumbnail_small_relative_url_str, str)
	assert thumbnail_small_relative_url_str.startswith('/images')
	
	assert thumbs_info_map.has_key('thumbnail_medium_relative_url_str')
	thumbnail_medium_relative_url_str = thumbs_info_map['thumbnail_medium_relative_url_str']
	assert isinstance(thumbnail_medium_relative_url_str, str)
	assert thumbnail_medium_relative_url_str.startswith('/images')
	
	assert thumbs_info_map.has_key('thumbnail_large_relative_url_str')
	thumbnail_large_relative_url_str = thumbs_info_map['thumbnail_large_relative_url_str']
	assert isinstance(thumbnail_large_relative_url_str, str)
	assert thumbnail_large_relative_url_str.startswith('/images')
	
	image_processing_results_map = {
		'image_id_str':    image_id_str,
		'thumbs_info_map': {
 			'thumbnail_small_relative_url_str':  thumbnail_small_relative_url_str,
			'thumbnail_medium_relative_url_str': thumbnail_medium_relative_url_str,
			'thumbnail_large_relative_url_str':  thumbnail_large_relative_url_str
		}
	}
	
	return image_processing_results_map